#include "Archive.h"
#include "GrayCoder.h"

Archive::Archive(Parameter& para){

	archiveSize = para.getPopulationSize();
	ranking = new SPEA2ranking(para);
	selector = new TournamentSelection();
	pena = new NSGAIIpenalty();
	
	//η
	objarchive.resize(archiveSize);
	for(int i=0;i<archiveSize;i++){
	  objarchive[i] = *(new Individual());	
	}
	
	//ե뤫
	if(para.getFinitial()){
	  ifstream fin;
	  fin.open(INITIAL_POPULATION_FILE);
	  if(!fin.is_open()){
	    cerr << __FILE__ << ":(" << __LINE__ << ") can't open file "
		 << INITIAL_POPULATION_FILE << endl;
	    exit(1);
	  }
	  file_init_pop(objarchive,fin, para);
	  fin.close();
	}
	
	//ɾ
	for(int i=0;i<archiveSize;i++){
	    objarchive[i].evaluation();
	}
	
}
		
Individual* Archive::getobjArchive(int i){
	return &objarchive[i];
}



void Archive::file_init_pop(vector<Individual>& dum_pop,ifstream& fin, Parameter& para)
{
  int i,j,k;
  double variable_s, closedMin, openMax, delta;
  int geneLength = para.getGeneLength();
  int designNum = para.getDesignNum();
  int designLength = para.getDesignLength();
  double *variable_value;
  char dum_char[100];
  int *gene, *gene_design;
  gene = new int[geneLength];
  gene_design = new int[designLength];
  variable_value = new double[designNum];
	  
  GrayCoder graycoder;
  
  fin.ignore(INT_MAX ,'\n');
  for(i = 0; i < dum_pop.size()&&fin.good(); ++i){
  fin >>dum_char;
  
  for(j = 0; j < designNum&&fin.good(); ++j){
      fin >> variable_s;
	  closedMin = para.getfunc()->getMin(j);
	  openMax = para.getfunc()->getMax(j);
	  delta = (openMax - closedMin) / ((double)(1U << para.getDesignLength()));
	  
	  // ˰ᤷ
	  if(variable_s < closedMin)
		variable_s = closedMin;
	  else if(variable_s >= openMax)
		variable_s = openMax - delta;
	  
	  // ¿ͤͤѴ (ݤϻͼθ)
	  int num = static_cast<int>((variable_s - closedMin) / delta + 0.5);
	  
	  // ɲgene_design[] ˳Ǽ
	  graycoder.grayencode(num, designLength, gene_design);
	  
	  for(k=0; k<designLength; k++){
		gene[j*designLength + k] = gene_design[k];
	  }
	  
	  variable_value[j]=variable_s;
	  if(fin.fail()){
		break;
	  }
    }
	
	dum_pop[i].setGene(gene);
    dum_pop[i].setdesignValue(variable_value);
	
  }
  
  delete [] gene;
  delete [] gene_design;
  delete [] variable_value;
}

Population Archive::getSearchPopulation()
{
  Population tmpPopulation;
  
  //ᥤƥˤ꿷õ콸Ĥ
  matingSelection(tmpPopulation);
  return tmpPopulation;
}

	
void Archive::updateArchive(Population& pop){
	exeranking(pop);
	penalty(pop);
	preserveArchive(pop);
}

//콸Ĥȥ֤򺮤ƥ󥭥
void Archive::exeranking(Population& pop){
    ranking->spea2ranking(pop.getPopulation(),objarchive);
}

//¹ԲǽΰϤ줿ΤФƥڥʥƥͿ
void Archive::penalty(Population& pop){
	//郎Ȥ
	if(Parameter::getInstance().getfunc()->getconstraintNum()>0){
		vector<Individual> poparc;
		poparc.resize(0);
		for(int i=0;i<archiveSize;i++){
			poparc.push_back(*pop.getPopulation(i));
		}
		for(int i=0;i<archiveSize;i++){
			poparc.push_back(objarchive[i]);
		}

		pena->penalty(poparc);
		
	
		for(int i=0;i<archiveSize;i++){
			pop.getPopulation(i)->setfitness(poparc[i].getfitness());
		}
		for(int i=0;i<archiveSize;i++){
			objarchive[i].setfitness(poparc[i+archiveSize].getfitness());
		}

	}
	//郎̵Ȥ
	else{
		//󥯤ͤ򤽤ΤޤfitnessȤ
		for(int i=0;i<archiveSize;i++){
			pop.getPopulation(i)->setfitness((double) pop.getPopulation(i)->getrank());
		}
		for(int i=0;i<archiveSize;i++){
			objarchive[i].setfitness((double) objarchive[i].getrank());
		}
		
	}	
}

//꡼¸
void Archive::preserveArchive(Population& pop){

	int tmpSize = pop.getPopulation().size()+objarchive.size();
	vector<Individual> tmp;
	tmp.resize(tmpSize);
	for(int i=0;i<pop.getPopulation().size();i++){
		tmp[i]=*pop.getPopulation(i);
		
	}
	for(int i=pop.getPopulation().size();i<objarchive.size()+pop.getPopulation().size();i++){
		tmp[i]=objarchive[i-pop.getPopulation().size()];
	}
	
	sort(tmp.begin(),tmp.end());
	fitness0num=0;
	//󥯣θΤο롥
	for(int i=0;i<tmp.size();i++){
		if(tmp[i].getfitness()==0){
			fitness0num++;
		}
	}

	objarchive.resize(archiveSize);
	
	if(fitness0num<=archiveSize){//ο֥ʲʤ
	  //fitnessΤ褤ˤΤޤޤ줾Υ֤
		for(int i=0;i<archiveSize;i++){
			objarchive[i]=tmp[i];
			//vararchive[i]=objarchive[i];
		}
	}		
	else{//ο֥ۤ
	  vector<Individual> over;	//֥ʾ
		over.resize(fitness0num);
		for(int i=0;i<fitness0num;i++){
			over[i]=tmp[i];	//Ǽ
		}
		truncation(over);//üڤ
	}
}

//ᥤƥ
void Archive::matingSelection(Population& pop){
	selector->select(*this,pop);
}

//üڤ
void Archive::truncation(vector<Individual>& fitness0){

    truncator = new Truncation(fitness0);//üڤꥤ󥹥
	truncator->objtruncation(fitness0); //Ūüڤ
	objarchive.resize(archiveSize);
	for(int i=0;i<archiveSize;i++){
		objarchive[i]=fitness0[i];//̤Ǽ
	}		
	delete truncator;	
}

